@@ -1,15 +1,16 @@
+from random import shuffle
 from typing import Any
 
 import factory
-from factory.declarations import LazyAttribute
 from factory.django import DjangoModelFactory
 from factory.fuzzy import BaseFuzzyAttribute
 from wagtail.core import blocks
 from wagtail.core.models import Page
+from wagtail.documents.models import Document
 from wagtail.images import get_image_model
-from wagtail_factories.blocks import ChooserBlockFactory
+from wagtail_factories.blocks import StreamFieldFactory
 
-from coop.utils.testdata import get_random_image
+from coop.utils.testdata import get_random_image, rbool
 
 Image = get_image_model()
 
@@ -19,32 +20,24 @@ class FuzzyImage(BaseFuzzyAttribute):
         return get_random_image()
 
 
+class FuzzyPage(BaseFuzzyAttribute):
+    def fuzz(self) -> Page:
+        return Page.objects.order_by('?').first()
+
+
 class FuzzyWords(factory.Faker):
-    def __init__(self, nb_words=1):
+    def __init__(self, nb_words: int = 1, maybe: bool = False):
+        self.maybe = maybe
         super().__init__('sentence', nb_words=nb_words)
 
     def evaluate(self, instance, step, extra) -> str:
+        if self.maybe and not rbool():
+            return ''
         # create a variable sentece, remove full stop
         value = super().evaluate(instance, step, extra)
         return value[:-1]
 
 
-# Might be redundant, above works for images, why not pages?
-class FuzzyPageChooserFactory(ChooserBlockFactory):
-    page = LazyAttribute(lambda _: Page.objects.order_by('?').first())
-
-    @classmethod
-    def _build(cls, model_class, page):
-        return page
-
-    @classmethod
-    def _create(cls, model_class, page):
-        return page
-
-    class Meta:
-        model = blocks.PageChooserBlock
-
-
 class ListLengthBlockFactory(factory.SubFactory):
     """
     Returns an n list of generated blocks
@@ -91,3 +84,31 @@ class RichTextBlockFactory(factory.Factory):
 
     class Meta:
         model = blocks.RichTextBlock
+
+
+class FuzzyStreamFieldFactory(StreamFieldFactory):
+    def __init__(self, factories, shuffle_blocks=True, **kwargs):
+        self.shuffle_blocks = shuffle_blocks
+        super().__init__(factories, **kwargs)
+
+    def generate(self, step, params):
+        retval = []
+        for block_name, block_factory in self.factories.items():
+            retval.append((block_name, block_factory()))
+        if self.shuffle_blocks:
+            shuffle(retval)
+        return retval
+
+
+class FuzzyDocument(BaseFuzzyAttribute):
+    def fuzz(self) -> Document:
+        return Document.objects.order_by('?').first()
+
+
+class FuzzyParagraphs(factory.Faker):
+    def __init__(self, num=1, **kwargs):
+        super().__init__('paragraphs', nb=num, **kwargs)
+
+    def evaluate(self, instance, step, extra) -> str:
+        value = super().evaluate(instance, step, extra)
+        return ''.join(f'<p>{p}</p>' for p in value)
